import * as rqErr from 'request-promise/errors';
export declare class LeaderInfo {
    leader: string;
    uptime: string;
    startTime: Date;
    static fill(js: any): LeaderInfo;
}
export declare class EtcValueNode {
    createIndex: number;
    key: string;
    modifiedIndex: number;
    value: string;
    dir: boolean;
    nodes: EtcValueNode[];
    static fromJson(js: any, val?: EtcValueNode): EtcValueNode;
}
export declare class EtcError {
    reqErr?: rqErr.RequestError;
    statusErr?: rqErr.StatusCodeError;
    transErr?: rqErr.TransformError;
    unknown?: any;
    static fromJson(err: any): EtcError;
}
export declare class EtcValue<T> {
    err?: EtcResponse;
    value?: T;
    static error<T>(res: any): EtcValue<T>;
    static value<T>(value: T): EtcValue<T>;
    isErr(): boolean;
    isOk(): boolean;
}
export declare class EtcResponse {
    action: string;
    node?: EtcValueNode;
    err?: EtcError;
    static error(err: any): EtcResponse;
    static fromJson(js: any): EtcResponse;
    isErr(): boolean;
    isOk(): boolean;
}
export declare class SelfState {
    url: string;
    err: any;
    name: string;
    id: string;
    state: string;
    startTime: Date;
    leaderInfo: LeaderInfo;
    recvAppendRequestCnt: number;
    sendAppendRequestCnt: number;
    static error(url: string, err: any): SelfState;
    static ok(url: string, val: string): SelfState;
    isOk(): boolean;
}
export declare class Config {
    urls: string[];
    reqTimeout: number;
    retries: number;
    waitTime: number;
    clusterId: string;
    appId: string;
    static start(argv: string[], app?: string): Config;
}
export declare class Etcd {
    private cfg;
    private connected?;
    private selfStateInActions;
    static create(cfg: Config): Etcd;
    constructor(cfg: Config);
    private request(method, url, options?);
    private rawRequest(method, url, options?);
    private urlParams(params, sep?);
    private bodyParams(params);
    private buildKeyUri(base, key);
    untilFirstConnect(): Promise<SelfState[]>;
    connect(): Promise<SelfState>;
    clusterState(): Promise<SelfState[]>;
    private resolvSelfStateInActions(url, s2);
    selfStat(url: string): Promise<SelfState>;
    private keyAction(method, key, options?);
    update(key: string, ttl: number): Promise<EtcResponse>;
    addQueue(key: string, val: string, ttl: number): Promise<EtcResponse>;
    mkdir(key: string): Promise<EtcResponse>;
    rmdir(key: string, params?: any): Promise<EtcResponse>;
    delete(key: string): Promise<EtcResponse>;
    list(key: string, params?: any): Promise<EtcValue<EtcValueNode[]>>;
    getRaw(key: string, params?: any, options?: any): Promise<EtcResponse>;
    getString(key: string, params?: any): Promise<EtcValue<string>>;
    getJson(key: string, params?: any): Promise<EtcValue<any>>;
    setRaw(key: string, val: string): Promise<EtcResponse>;
    setJson(key: string, val: any): Promise<EtcResponse>;
}
export default Etcd;
