"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rq = require("request-promise");
class LeaderInfo {
    static fill(js) {
        let ret = new LeaderInfo();
        ret.leader = js['leader'];
        ret.uptime = js['uptime'];
        ret.startTime = new Date(js['startTime']);
        return ret;
    }
}
exports.LeaderInfo = LeaderInfo;
class Dispatcher {
    constructor() {
        // console.log('Dispatcher:constructor')
        this.promise = new Promise((s, j) => {
            // console.log('Dispatcher:', s, j)
            this.fnResolv = s;
            this.fnReject = j;
        });
    }
    static create() {
        return new Dispatcher();
    }
    resolve(t) {
        this.fnResolv(t);
    }
    reject(t) {
        this.fnReject(t);
    }
}
class EtcValueNode {
    constructor() {
        this.dir = false;
        this.nodes = null;
    }
    static fromJson(js, val = null) {
        if (!val) {
            val = new EtcValueNode();
        }
        val.createIndex = js['createIndex'];
        val.key = js['key'];
        val.modifiedIndex = js['modifiedIndex'];
        val.value = js['value'];
        val.dir = js['dir'] || false;
        if (val.dir) {
            val.nodes = (js['nodes'] || []).map((n) => EtcValueNode.fromJson(n));
        }
        return val;
    }
}
exports.EtcValueNode = EtcValueNode;
class EtcError {
    static fromJson(err) {
        let ee = new EtcError();
        // if (typeof (err) == 'RequestError') {
        //   return ee;
        // }
        if (typeof (err.statusCode) == 'number' && err.statusCode != 200) {
            ee.statusErr = err;
            return ee;
        }
        // if (typeof (err) == 'TransformError') {
        //   return ee;
        // }
        ee.unknown = err;
        return ee;
    }
}
exports.EtcError = EtcError;
class EtcValue {
    constructor() {
        this.err = null;
        this.value = null;
    }
    static error(res) {
        let ej = new EtcValue();
        ej.err = res;
        // console.log('ERROR', res)
        // ej.err = EtcErrorFactory(res)
        return ej;
    }
    static value(value) {
        let ej = new EtcValue();
        ej.value = value;
        return ej;
    }
    isErr() {
        return !!this.err;
    }
    isOk() {
        return !this.isErr();
    }
}
exports.EtcValue = EtcValue;
// function EtcNodeFactory(js: any) : EtcValueNode {
//   return EtcValueNode.fromJson(js)
// }
class EtcResponse {
    static error(err) {
        let res = new EtcResponse();
        res.action = 'error';
        res.err = EtcError.fromJson(err);
        return res;
    }
    static fromJson(js) {
        let res = new EtcResponse();
        res.action = js['action'];
        res.node = EtcValueNode.fromJson(js['node']);
        return res;
    }
    isErr() {
        return this.action == 'error';
    }
    isOk() {
        return !this.isErr();
    }
}
exports.EtcResponse = EtcResponse;
// function EtcResponseFactory(js: any) : EtcResponse {
//   // if (js['errorCode']) {
//   //   return EtcError.fromJson(js)
//   // }
//   if (js['action']) {
//     return EtcResponse.fromJson(js)
//   }
//   return null
// }
// function EtcErrorFactory(js: any) : any {
//   if (js['action']) {
//     return EtcResponse.fromJson(js)
//   }
//   return null
// }
class SelfState {
    constructor() {
        this.err = null;
    }
    static error(url, err) {
        let ret = new SelfState();
        ret.url = url;
        ret.err = err;
        return ret;
    }
    static ok(url, val) {
        let ret = new SelfState();
        ret.url = url;
        let json = JSON.parse(val);
        ret.name = json['name'];
        ret.id = json['id'];
        ret.state = json['state'];
        ret.startTime = new Date(json['startTime']);
        ret.leaderInfo = LeaderInfo.fill(json['leaderInfo']);
        ret.recvAppendRequestCnt = json['recvAppendRequestCnt'];
        ret.sendAppendRequestCnt = json['sendAppendRequestCnt'];
        return ret;
    }
    isOk() {
        return this.err == null && this.id.length != 0;
    }
}
exports.SelfState = SelfState;
class Config {
    constructor() {
        this.urls = [];
        this.reqTimeout = 500; // msec
        this.retries = 3;
        this.waitTime = 250; // ms
        this.clusterId = null;
        this.appId = null;
    }
    static start(argv, app = null) {
        let ret = new Config();
        let ofs = argv.indexOf('--etcd-cluster-id');
        if (ofs >= 0) {
            ret.clusterId = argv[ofs + 1];
        }
        ofs = argv.indexOf('--etcd-app-id');
        if (ofs >= 0) {
            ret.appId = argv[ofs + 1];
        }
        else {
            ret.appId = app;
        }
        for (ofs = argv.indexOf('--etcd-url'); ofs >= 0; ofs = argv.indexOf('--etcd-url', ofs + 1)) {
            ret.urls.push(argv[ofs + 1]);
        }
        if (ret.urls.length == 0) {
            ret.urls.push('http://localhost:2379');
        }
        ofs = argv.indexOf('--etcd-retries');
        if (ofs >= 0) {
            ret.retries = parseInt(argv[ofs + 1], 10);
        }
        ofs = argv.indexOf('--etcd-wait-time');
        if (ofs >= 0) {
            ret.waitTime = parseInt(argv[ofs + 1], 10);
        }
        ofs = argv.indexOf('--etcd-req-timeout');
        if (ofs >= 0) {
            ret.reqTimeout = parseInt(argv[ofs + 1], 10);
        }
        return ret;
    }
}
exports.Config = Config;
class Etcd {
    constructor(cfg) {
        this.connected = null;
        this.selfStateInActions = {};
        this.cfg = cfg;
    }
    static create(cfg) {
        return new Etcd(cfg);
    }
    request(method, url, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let c = yield this.connect();
            if (!c.isOk()) {
                console.error('Request-REJECT no valid connection');
                return Promise.reject(c);
            }
            try {
                return yield this.rawRequest(method, `${c.url}${url}`, options);
            }
            catch (e) {
                if (e.name == 'StatusCodeError') {
                    return Promise.reject(e);
                }
                console.error('Reconnected:', typeof e, e.name, e);
                this.connected = null; // reconnect etcd
                return this.request(method, url, options);
            }
        });
    }
    rawRequest(method, url, options = {}) {
        let my = {
            method: method,
            timeout: this.cfg.reqTimeout,
            uri: url
        };
        options = Object.assign(my, options);
        // console.log(options)
        return rq(url, options);
    }
    urlParams(params, sep = '') {
        let paramsStr = '';
        for (let key in params) {
            paramsStr += `${sep}${key}=${params[key]}`;
            sep = '&';
        }
        return paramsStr;
    }
    bodyParams(params) {
        return {
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            body: this.urlParams(params)
        };
    }
    buildKeyUri(base, key) {
        let url;
        let app = this.cfg.appId || '';
        if (app.length != 0) {
            app += '/';
        }
        if (this.cfg.clusterId) {
            url = `${base}/${app}${this.cfg.clusterId}/${key}`;
        }
        else {
            url = `${base}/${app}${key}`;
        }
        return url;
    }
    untilFirstConnect() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((res, rej) => {
                // console.log('ufc-1')
                let resolved = false;
                let result = [];
                // console.log('ufc-2')
                this.cfg.urls.forEach((url) => {
                    // console.log('ufc-3')
                    this.selfStat(url).then((value) => {
                        // console.log('ufc-4')
                        if (resolved) {
                            return;
                        }
                        result.push(value);
                        if (value.isOk()) {
                            // console.log('ufc-5')
                            resolved = true;
                            res(result);
                        }
                        else {
                            // console.log('ufc-6')
                            if (result.length == this.cfg.urls.length) {
                                // console.log('ufc-7')
                                resolved = true;
                                res(result);
                            }
                        }
                    });
                });
            });
        });
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('meno-1')
            if (!this.connected) {
                let ret = null;
                for (let retry = 0; retry < this.cfg.retries; ++retry) {
                    // console.log('meno-2')
                    ret = yield this.untilFirstConnect();
                    // console.log('meno-3')
                    let foundOk = ret.find((s2) => s2.isOk());
                    if (foundOk) {
                        // console.log('meno-4')
                        this.connected = foundOk;
                        return Promise.resolve(foundOk);
                    }
                    console.log('Retry-Connect:', retry, this.cfg.waitTime);
                    yield new Promise((res, rej) => {
                        setTimeout(res, this.cfg.waitTime);
                    });
                }
                return Promise.resolve(SelfState.error('all', ret));
            }
            else {
                return Promise.resolve(this.connected);
            }
        });
    }
    clusterState() {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('this.cfg.urls:', this.cfg.urls)
            return Promise.all(this.cfg.urls.map((url) => this.selfStat(url)));
        });
    }
    resolvSelfStateInActions(url, s2) {
        let local = this.selfStateInActions[url];
        delete this.selfStateInActions[url];
        local.slice(1).forEach((ssia) => {
            ssia.resolve(s2);
        });
        return s2;
    }
    selfStat(url) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('selfStat:', url)
            if (!this.selfStateInActions[url]) {
                this.selfStateInActions[url] = [];
            }
            let dispatcher = Dispatcher.create();
            this.selfStateInActions[url].push(dispatcher);
            if (this.selfStateInActions[url].length > 1) {
                // console.log('selfStat-Double:', url)
                return dispatcher.promise;
            }
            return new Promise((res, rej) => __awaiter(this, void 0, void 0, function* () {
                //  console.log('P-Enter')
                try {
                    let ret = yield this.rawRequest('GET', `${url}/v2/stats/self`);
                    //  console.log('selfstate: get ok:', url, ret)
                    res(this.resolvSelfStateInActions(url, SelfState.ok(url, ret)));
                }
                catch (err) {
                    //  console.log('selfstate: get false:', url, err)
                    res(this.resolvSelfStateInActions(url, SelfState.error(url, err)));
                }
            }));
        });
    }
    keyAction(method, key, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let uri = this.buildKeyUri('/v2/keys', key);
            try {
                let ret = yield this.request(method, uri, options);
                // if (method == 'PUT') {
                //   console.log('OK-PUT:', uri, ret)
                // }
                return Promise.resolve(EtcResponse.fromJson(JSON.parse(ret)));
            }
            catch (err) {
                // if (method == 'PUT') {
                //   console.log('ERR-PUT:', uri, err)
                // }
                return Promise.resolve(EtcResponse.error(err));
            }
        });
    }
    update(key, ttl) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyAction('PUT', key, this.bodyParams({
                'ttl': ttl,
                'refresh': true,
                'prevExist': true
            }));
        });
    }
    addQueue(key, val, ttl) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyAction('POST', key, this.bodyParams({ 'value': val, 'ttl': ttl }));
        });
    }
    mkdir(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyAction('PUT', key, this.bodyParams({ dir: true }));
        });
    }
    rmdir(key, params = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyAction('DELETE', `${key}${this.urlParams(Object.assign({ dir: true }, params), '?')}`);
        });
    }
    delete(key) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyAction('DELETE', key);
        });
    }
    list(key, params = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('list:', `${key}${this.urlParams(params, '?')}`)
            let list = yield this.keyAction('GET', `${key}${this.urlParams(params, '?')}`);
            if (list.isErr()) {
                return Promise.resolve(EtcValue.error(list));
            }
            if (!list.node.dir) {
                return Promise.resolve(EtcValue.error('not a directory'));
            }
            return Promise.resolve(EtcValue.value(list.node.nodes)); // empty directory
        });
    }
    getRaw(key, params = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            // console.log('get:', `${key}${this.urlParams(params, '?')}`)
            return this.keyAction('GET', `${key}${this.urlParams(params, '?')}`, options);
        });
    }
    getString(key, params = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let ret = yield this.getRaw(key, params);
                if (ret.isErr()) {
                    return Promise.resolve(EtcValue.error(ret));
                }
                return Promise.resolve(EtcValue.value(ret.node.value));
            }
            catch (err) {
                return Promise.resolve(EtcValue.error(err));
            }
        });
    }
    getJson(key, params = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let ret = yield this.getRaw(key, params);
                if (ret.isErr()) {
                    return Promise.resolve(EtcValue.error(ret));
                }
                return Promise.resolve(EtcValue.value(JSON.parse(ret.node.value)));
            }
            catch (err) {
                return Promise.resolve(EtcValue.error(err));
            }
        });
    }
    setRaw(key, val) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let ret = yield this.keyAction('PUT', key, this.bodyParams({ value: val }));
                return Promise.resolve(ret);
            }
            catch (err) {
                return Promise.resolve(EtcResponse.error(err));
            }
        });
    }
    setJson(key, val) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.setRaw(key, JSON.stringify(val));
        });
    }
}
exports.Etcd = Etcd;
exports.default = Etcd;
//# sourceMappingURL=etcd.js.map