"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Uuid = require("node-uuid");
const os = require("os");
const path = require("path");
class WaitMaster {
    constructor(key, etc, ttl = 30000, saveTtl = 1000, startCb, stopCb) {
        this.reqStop = false;
        this.stopped = false;
        this.stopAction = null;
        this.master = false;
        this.startCb = null;
        this.stopCb = null;
        this.currentAliveTimeOut = null;
        this.currentWait = null;
        this.etc = etc;
        this.key = key;
        this.ttl = ttl;
        this.saveTtl = saveTtl;
        this.startCb = startCb;
        this.stopCb = stopCb;
    }
    static create(key, etc, ttl, saveTtl, start, stop = null) {
        return __awaiter(this, void 0, void 0, function* () {
            let ret = new WaitMaster(key, etc, ttl, saveTtl, start, stop);
            try {
                yield etc.mkdir(key);
            }
            catch (err) {
                if (err.statusCode != 403) {
                    console.log(err); // Error:  105: Key already exists (/menox)
                    return Promise.reject(`can not create directory:${key}`);
                }
            }
            ret.me = `${os.hostname()}-${process.pid}-${Uuid.v4().toString()}`;
            setTimeout(ret.startWaitChange.bind(ret), 0);
            try {
                let aq = yield etc.addQueue(key, ret.me, ret.ttl + ret.saveTtl);
                ret.aliveQueue = `${key}/${path.basename(aq['node']['key'])}`;
                ret.startAliveQueue();
            }
            catch (e) {
                return Promise.reject(e);
            }
            return Promise.resolve(ret);
        });
    }
    renewTimeout() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.reqStop) {
                return;
            }
            try {
                // console.log(this.aliveQueue)
                yield this.etc.update(this.aliveQueue, this.ttl + this.saveTtl);
            }
            catch (e) {
                console.error('update alive failed');
                return this.stop();
            }
            this.startAliveQueue();
        });
    }
    startAliveQueue() {
        return __awaiter(this, void 0, void 0, function* () {
            this.currentAliveTimeOut = setTimeout(this.renewTimeout.bind(this), this.ttl);
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.reqStop) {
                return Promise.resolve('stop is running');
            }
            this.reqStop = true;
            if (this.currentAliveTimeOut) {
                clearTimeout(this.currentAliveTimeOut);
            }
            // console.log('ReqStop:', this.me, this.aliveQueue)
            yield this.etc.delete(this.aliveQueue);
            // this.currentWait && this.currentWait.cancel()
        });
    }
    doStop(masterId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.master = false;
            this.stopped = true;
            if (this.stopAction) {
                this.stopAction();
                this.stopAction = null;
            }
            // console.log('Stop:', this.me, this.aliveQueue, masterId)
        });
    }
    checkAmIMaster() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let master = yield this.etc.list(this.key, { recursive: true, sorted: true });
                // console.log('master:', this.key, master)
                if (master.isErr()) {
                    return Promise.resolve(master);
                }
                let masterId = null;
                if (master.value.length != 0) {
                    masterId = `${this.key}/${path.basename(master.value[0]['key'])}`;
                }
                // console.log('master:', master)
                if (this.master && masterId != this.aliveQueue) {
                    this.doStop(masterId);
                    return Promise.resolve('got slave');
                }
                // let value = await this.etc.getString(masterId)
                if (masterId == this.aliveQueue) {
                    if (this.master) {
                        return Promise.resolve('im the master');
                    }
                    // this.masterId = masterId
                    // console.log('Master:', this.me, this.aliveQueue)
                    this.master = true;
                    this.stopAction = this.stopCb;
                    this.startCb();
                    return Promise.resolve('got master');
                }
                if (this.reqStop && !this.master) {
                    this.stopped = true;
                    return Promise.resolve('slaved stopped');
                }
                return Promise.resolve('im a slave');
            }
            catch (e) {
                console.error(e);
                return Promise.reject(e);
            }
        });
    }
    startWaitChange() {
        return __awaiter(this, void 0, void 0, function* () {
            let waitIndex = null;
            while (!this.stopped) {
                try {
                    let params = { 'wait': true, 'recursive': true };
                    if (waitIndex) {
                        params['waitIndex'] = waitIndex;
                    }
                    // let master = this.master;
                    // master && console.log('wait...', this.me, this.aliveQueue)
                    this.currentWait = this.etc.getRaw(this.key, params, { timeout: 3600000 });
                    let ret = yield this.currentWait;
                    // master && console.log('await', this.me, this.aliveQueue)
                    waitIndex = ret['node']['modifiedIndex'] + 1;
                    ret = yield this.checkAmIMaster();
                    // master && console.log('postMaster:', ret, this.aliveQueue)
                }
                catch (e) {
                    console.error(e);
                }
            }
        });
    }
}
exports.WaitMaster = WaitMaster;
//# sourceMappingURL=wait_master.js.map