"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Uuid = require("node-uuid");
const etcd = require("../lib/index");
/*
function param(arr: string[], uuid: string): string[] {
  return arr.concat(['--etcd-cluster-id', uuid, '--etcd-url', 'http://localhost:2379']);
}
*/
describe('etcd', function () {
    this.timeout(2000);
    before(() => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        yield etc.connect();
        console.log('etcd Cluster Booted');
        return Promise.resolve('done');
    }));
    it('AsyncPromise Blocking', () => __awaiter(this, void 0, void 0, function* () {
        let value = 1;
        let out = new Promise((r, j) => __awaiter(this, void 0, void 0, function* () {
            yield new Promise((rr, jj) => { setTimeout(rr, 10); });
            ++value;
        }));
        out = null; // WTF Lint
        chai_1.assert.equal(value, 1);
        yield new Promise((rr, jj) => { setTimeout(rr, 15); });
        chai_1.assert.equal(value, 2);
        return Promise.resolve('done');
    }));
    it('selfStat one good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(1, ret.length);
        // console.log(ret[0])
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat two same good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(2, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        chai_1.assert.equal(ret[1].isOk(), true);
        chai_1.assert.equal(ret[1].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat same good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(4, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[0].url, 'http://localhost:2379');
        return Promise.resolve('done');
    }));
    it('selfStat connect', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true, 'isok');
        chai_1.assert.equal(ret.url, 'http://localhost:2379', 'url');
        return Promise.resolve('done');
    }));
    it('selfStat clusterState bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://169.254.99.94'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(1, ret.length);
        chai_1.assert.equal(ret[0].isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat connect bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://169.254.99.94',
            '--etcd-url', 'http://169.254.99.95'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat connect good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat connect two good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat connect two good', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.55.99',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.connect();
        chai_1.assert.equal(ret.isOk(), true);
        return Promise.resolve('done');
    }));
    it('selfStat one good one bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.99.94'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(2, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[1].isOk(), false);
        return Promise.resolve('done');
    }));
    it('selfStat two good one bad', () => __awaiter(this, void 0, void 0, function* () {
        let wc = etcd.Config.start([
            '--etcd-req-timeout', '50',
            '--etcd-url', 'http://localhost:2379',
            '--etcd-url', 'http://169.254.99.94',
            '--etcd-url', 'http://127.0.0.1:2379'
        ]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.clusterState();
        chai_1.assert.equal(3, ret.length);
        chai_1.assert.equal(ret[0].isOk(), true);
        chai_1.assert.equal(ret[1].isOk(), false);
        chai_1.assert.equal(ret[2].isOk(), true);
        return Promise.resolve('done');
    }));
    it('mkdir', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.mkdir('meno');
        chai_1.assert.equal(ret.isErr(), false);
        ret = yield etc.mkdir('meno');
        chai_1.assert.equal(ret.isErr(), true);
        let lst = yield etc.list('meno');
        chai_1.assert.equal(lst.isErr(), false);
        chai_1.assert.equal(lst.value.length, 0);
        ret = yield etc.rmdir('meno');
        chai_1.assert.equal(ret.isOk(), true);
    }));
    it('rmdir', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.Etcd.create(wc);
        yield etc.mkdir('meno');
        yield etc.mkdir('meno/geheim');
        yield etc.mkdir('meno/geheim/ganz');
        let ret = yield etc.rmdir('meno', { recursive: true });
        chai_1.assert.equal(ret.isOk(), true);
        yield etc.mkdir('meno');
        let lst = yield etc.list('meno');
        chai_1.assert.equal(lst.isErr(), false);
        chai_1.assert.equal(lst.value.length, 0);
        ret = yield etc.rmdir('meno', { recursive: true });
        chai_1.assert.equal(ret.isOk(), true);
    }));
    it('get/setRaw', () => __awaiter(this, void 0, void 0, function* () {
        let uuid = Uuid.v4().toString();
        let wc = etcd.Config.start(['--etcd-cluster-id', uuid]);
        let etc = etcd.Etcd.create(wc);
        let ret = yield etc.getRaw('hammer/murk');
        chai_1.assert.equal(ret.isErr(), true);
        ret = yield etc.setRaw('hammer/murk', 'Hello World');
        chai_1.assert.equal(ret.isErr(), false);
        ret = yield etc.getRaw('hammer/murk');
        chai_1.assert.equal(ret.isOk(), true);
        // console.log(ret.node)
        chai_1.assert.equal(ret.node.value, 'Hello World');
    }));
});
//# sourceMappingURL=etcd-test.js.map